// NumberBaseballRobot.js

(function(ext) {
    var device = null;
	var _rxBuf = [];
    
	var levels = {
		HIGH:1,
		LOW:0
	};
    
	ext.resetAll = function(){};
	
	ext.runArduino = function(){
		
	};
	
	ext.digitalWrite = function(pin,level) {
        device.send([pin, levels[level]])
    };
	
	var _level = 0;
	
	ext.blink = function(){
		device.send([0x22, 0x23])
	}

    function processData(bytes) {
		var len = bytes.length;
		if (_rxBuf.length > 30) {
		  _rxBuf = [];
		}
		for (var index = 0; index < bytes.length; index++) {
			var c = bytes[ index ];
			_rxBuf.push(c);
			if (_rxBuf.length >= 2) {
				if (_rxBuf[ _rxBuf.length - 1 ] == 0x55 && _rxBuf[ _rxBuf.length - 2 ] == 0xff) {
					_isParseStart = true;
					_isParseStartIndex = _rxBuf.length - 2;
				}
				if (_rxBuf[ _rxBuf.length - 1 ] == 0xa && _rxBuf[ _rxBuf.length - 2 ] == 0xd && _isParseStart) {
					_isParseStart = false;

					var position = _isParseStartIndex + 2;
					var extId = _rxBuf[ position ];
					position++;
					var type = _rxBuf[ position ];
					position++;
					//1 byte 2 float 3 short 4 len+string 5 double
					var value;
					switch (type) {
						case 1: {
								value = _rxBuf[ position ];
								position++;
							}
							break;
						case 2: {
								value = readFloat(_rxBuf, position);
								position += 4;
								if (value < -255 || value > 1023) {
									value = 0;
								}
							}
							break;
						case 3: {
								value = readInt(_rxBuf, position, 2);
								position += 2;
							}
							break;
						case 4: {
								var l = _rxBuf[ position ];
								position++;
								value = readString(_rxBuf, position, l);
							}
						break;
						case 5: {
								value = readDouble(_rxBuf, position);
								position += 4;
							}
							break;
						case 6:
								value = readInt(_rxBuf, position, 4);
								position += 4;
							break;
					}
					if (type <= 6) {
						responseValue(extId, value);
					} else {
						responseValue();
					}
					_rxBuf = [];
				}
			}
		}
	}

    // Extension API interactions
    var potentialDevices = [];
    ext._deviceConnected = function(dev) {
        potentialDevices.push(dev);

        if (!device) {
            tryNextDevice();
        }
    }
	
	function tryNextDevice() {
        // If potentialDevices is empty, device will be undefined.
        // That will get us back here next time a device is connected.
        device = potentialDevices.shift();
        if (device) {
            device.open({ stopBits: 0, bitRate: 115200, ctsFlowControl: 0 }, deviceOpened);
        }
    }
	
	var watchdog = null;
	
    function deviceOpened(dev) {
		if (!dev) {
			// Opening the port failed.
			tryNextDevice();
			return;
		}
		device.set_receive_handler('arduino', processData);
	};
	
    ext._deviceRemoved = function(dev) {
        if(device != dev) return;
        device = null;
    };

    ext._shutdown = function() {
        if(device) device.close();
        device = null;
    };

    
	ext._getStatus = function () {
		if (!device) return {status: 1, msg: 'Arduino disconnected'};
		if (watchdog) return {status: 1, msg: 'Probing for Arduino'};
		return {status: 2, msg: 'Arduino connected'};
	}

    var descriptor = {};
	ScratchExtensions.register('NumberBaseballRobot', descriptor, ext, {type: 'serial'});
})({});
