/*
      LedControl.cpp - A library for controling Leds with a MAX7219/MAX7221
      Copyright (c) 2007 Eberhard Fahle

      Permission is hereby granted, free of charge, to any person
      obtaining a copy of this software and associated documentation
      files (the "Software"), to deal in the Software without
      restriction, including without limitation the rights to use,
      copy, modify, merge, publish, distribute, sublicense, and/or sell
      copies of the Software, and to permit persons to whom the
      Software is furnished to do so, subject to the following
      conditions:

      This permission notice shall be included in all copies or
      substantial portions of the Software.

      THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
      EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
      OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
      NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
      HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
      WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
      FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
      OTHER DEALINGS IN THE SOFTWARE.
*/


#include "Number_Baseball_Robot.h"

//the opcodes for the MAX7221 and MAX7219
#define OP_NOOP   0
#define OP_DIGIT0 1
#define OP_DIGIT1 2
#define OP_DIGIT2 3
#define OP_DIGIT3 4
#define OP_DIGIT4 5
#define OP_DIGIT5 6
#define OP_DIGIT6 7
#define OP_DIGIT7 8
#define OP_DECODEMODE  9
#define OP_INTENSITY   10
#define OP_SCANLIMIT   11
#define OP_SHUTDOWN    12
#define OP_DISPLAYTEST 15

#define NBR_MTX 2

#define Number_Position_1  3
#define Number_Position_2  7
#define Number_Position_3  11


int Key_Data[10] = {HIGH, HIGH, HIGH, HIGH, HIGH, HIGH, HIGH, HIGH, HIGH, HIGH};
int Key_Data_Before[10] = {HIGH, HIGH, HIGH, HIGH, HIGH, HIGH, HIGH, HIGH, HIGH, HIGH};


Number_Baseball_Robot::Number_Baseball_Robot(int dataPin, int clkPin, int csPin) {
  int numDevices = NBR_MTX;
  LEDCONTROL_SPI_MOSI = dataPin;
  LEDCONTROL_SPI_CLK = clkPin;
  LEDCONTROL_SPI_CS = csPin;
  if (numDevices <= 0 || numDevices > 8 )
    numDevices = 8;
  maxDevices = numDevices;
  pinMode(LEDCONTROL_SPI_MOSI, OUTPUT);
  pinMode(LEDCONTROL_SPI_CLK, OUTPUT);
  pinMode(LEDCONTROL_SPI_CS, OUTPUT);
  digitalWrite(LEDCONTROL_SPI_CS, HIGH);
  LEDCONTROL_SPI_MOSI = dataPin;
  for (int i = 0; i < 64; i++)
    status[i] = 0x00;
  for (int i = 0; i < maxDevices; i++) {
    spiTransfer(i, OP_DISPLAYTEST, 0);
    //scanlimit is set to max on startup
    setScanLimit(i, 7);
    //decode is done in source
    spiTransfer(i, OP_DECODEMODE, 0);
    clearDisplay(i);
    //we go into shutdown-mode on startup
    shutdown(i, true);
  }
}


void Number_Baseball_Robot::begin() {
  for (int i = 0; i < NBR_MTX; i++) {
    shutdown(i, false);
    /* Set the brightness to a medium values */
    setIntensity(i, 7);
    /* and clear the display */
    clearDisplay(i);
  }
}


int Number_Baseball_Robot::readKey(int address, int Button_Pin[]) {
  Key_Data[address] = digitalRead(Button_Pin[address]);
  delay(10);
  if (Key_Data[address] == LOW && Key_Data[address] != Key_Data_Before[address])  {
    Key_Data_Before[address] = Key_Data[address];
    return 1;
  }
  else if (Key_Data[address] == HIGH && Key_Data[address] != Key_Data_Before[address])   {
    Key_Data_Before[address] = Key_Data[address];
  }
  else;
  return 0;
}


void Number_Baseball_Robot::Column(byte x, byte data) {

  //select which matrix depending on the x coord
  byte address;
  if (x >= 0 && x <= 7)   {
    address = 1;
  }
  else if (x >= 8 && x <= 15)  {
    address = 0;
    x = x - 8;
  }
  setRow(address, x, data);
}


void Number_Baseball_Robot::Display_3_Text(byte x, byte y, byte number) {

  //select which matrix depending on the x coord
  byte data;
  for (int i = 0; i < 3; i++) {
    data = mytinyfont[number][i];
    Column(x + i, data << y);
  }
  delay(5);
}


void Number_Baseball_Robot::ExpressionDisp (byte Disp, byte number) {

  //select which matrix depending on the x coord
  byte data;
  if (Disp == 0) {
    for (int i = 0; i < 8; i++) {
      data = myRobotExpression[number][i];
      Column(7 - i, data);
    }
  }
  else {
    for (int i = 0; i < 8; i++) {
      data = myRobotExpression[number][i];
      Column(8 + i, data);
    }
  }
}

void Number_Baseball_Robot::UnderLine (byte x) {

  //select which matrix depending on the x coord
  for (int i = 0; i < 3; i++) {
    Point(x + i, 0, true);
  }
  delay(5);
}

void Number_Baseball_Robot::UnderLineErase(byte x) {

  //select which matrix depending on the x coord
  for (int i = 0; i < 3; i++) {
    Point(x + i, 0, false);
  }
  delay(5);
}
void Number_Baseball_Robot::Point (byte x, byte y, byte val) {

  //select which matrix depending on the x coord
  byte address;
  if (x >= 0 && x <= 7)   {
    address = 1;
  }
  else if (x >= 8 && x <= 15)  {
    address = 0;
    x = x - 8;
  }
  setLed(address, x, y, val);
}


void Number_Baseball_Robot::Display_Clear() {
  clearDisplay(0);
  clearDisplay(1);
}


void Number_Baseball_Robot::Display_Fail() {
  clearDisplay(0);
  clearDisplay(1);
  Display_3_Text(0, 1, 'L' - 54);
  Display_3_Text(4, 1, 'O' - 54);
  Display_3_Text(8, 1, 'S' - 54);
  Display_3_Text(12, 1, 'E' - 54);
}

void Number_Baseball_Robot::Display_Strike_Ball(int Strike_data, int Ball_data) {

  clearDisplay(0);
  clearDisplay(1);
  Display_3_Text(0, 1, Strike_data);
  Display_3_Text(4, 1, 'S' - 54);
  Display_3_Text(8, 1, Ball_data);
  Display_3_Text(12, 1, 'B' - 54);

}

void Number_Baseball_Robot::Display_Strike(int Strike_data) {

  clearDisplay(0);
  clearDisplay(1);
  Display_3_Text(4, 1, Strike_data);
  Display_3_Text(8, 1, 'S' - 54);

}

void Number_Baseball_Robot::Display_Ball(int Ball_data) {
  clearDisplay(0);
  clearDisplay(1);
  Display_3_Text(4, 1, Ball_data);
  Display_3_Text(8, 1, 'B' - 54);
}

void Number_Baseball_Robot::Display_Finish() {
  clearDisplay(0);
  clearDisplay(1);
  Display_3_Text(Number_Position_1, 1, 'W' - 54);
  Display_3_Text(Number_Position_2, 1, 'I' - 54);
  Display_3_Text(Number_Position_3, 1, 'N' - 54);
}

void Number_Baseball_Robot::Display_OUT() {
  clearDisplay(0);
  clearDisplay(1);
  Display_3_Text(Number_Position_1, 1, 'O' - 54);
  Display_3_Text(Number_Position_2, 1, 'U' - 54);
  Display_3_Text(Number_Position_3, 1, 'T' - 54);
}

void Number_Baseball_Robot::Display_Inning(int inning) {
  Display_3_Text(Number_Position_1, 1, inning );

  // 1~3은 영어로 서수를 줄여서 표기하는 방법이 다름 (First Second Third)
  if (inning == 1) {               // First
    Display_3_Text(Number_Position_2, 1, 'S' - 54 );
    Display_3_Text(Number_Position_3, 1, 'T' - 54 );
  }
  else if (inning == 2) {          // Second
    Display_3_Text(Number_Position_2, 1, 'N' - 54 );
    Display_3_Text(Number_Position_3, 1, 'D' - 54 );
  }
  else if (inning == 3) {          // Third
    Display_3_Text(Number_Position_2, 1, 'R' - 54 );
    Display_3_Text(Number_Position_3, 1, 'D' - 54 );
  }
  else {                                // 나머지 숫자는 th로 표기함
    Display_3_Text(Number_Position_2, 1, 'T' - 54 );
    Display_3_Text(Number_Position_3, 1, 'H' - 54 );
  }
}

int Number_Baseball_Robot::getDeviceCount() {
  return maxDevices;
}

void Number_Baseball_Robot::shutdown(int addr, bool b) {
  if (addr < 0 || addr >= maxDevices)
    return;
  if (b)
    spiTransfer(addr, OP_SHUTDOWN, 0);
  else
    spiTransfer(addr, OP_SHUTDOWN, 1);
}

void Number_Baseball_Robot::setScanLimit(int addr, int limit) {
  if (addr < 0 || addr >= maxDevices)
    return;
  if (limit >= 0 || limit < 8)
    spiTransfer(addr, OP_SCANLIMIT, limit);
}

void Number_Baseball_Robot::setIntensity(int addr, int intensity) {
  if (addr < 0 || addr >= maxDevices)
    return;
  if (intensity >= 0 || intensity < 16)
    spiTransfer(addr, OP_INTENSITY, intensity);

}

void Number_Baseball_Robot::clearDisplay(int addr) {
  int offset;

  if (addr < 0 || addr >= maxDevices)
    return;
  offset = addr * 8;
  for (int i = 0; i < 8; i++) {
    status[offset + i] = 0;
    spiTransfer(addr, i + 1, status[offset + i]);
  }
}

void Number_Baseball_Robot::clearAll() {
  for (int i = 0; i < maxDevices; i++) clearDisplay(i);
}

void Number_Baseball_Robot::setLed(int addr, int row, int column, boolean state) {
  int offset;
  byte val = 0x00;

  if (addr < 0 || addr >= maxDevices)
    return;
  if (row < 0 || row > 7 || column < 0 || column > 7)
    return;
  offset = addr * 8;
  val = B10000000 >> column;
  if (state)
    status[offset + row] = status[offset + row] | val;
  else {
    val = ~val;
    status[offset + row] = status[offset + row] & val;
  }
  spiTransfer(addr, row + 1, status[offset + row]);
}

void Number_Baseball_Robot::setRow(int addr, int row, byte value) {
  int offset;
  if (addr < 0 || addr >= maxDevices)
    return;
  if (row < 0 || row > 7)
    return;
  offset = addr * 8;
  status[offset + row] = value;
  spiTransfer(addr, row + 1, status[offset + row]);
}

void Number_Baseball_Robot::setColumn(int addr, int col, byte value) {
  byte val;

  if (addr < 0 || addr >= maxDevices)
    return;
  if (col < 0 || col > 7)
    return;
  for (int row = 0; row < 8; row++) {
    val = value >> (7 - row);
    val = val & 0x01;
    setLed(addr, row, col, val);
  }
}

void Number_Baseball_Robot::setDigit(int addr, int digit, byte value, boolean dp) {
  int offset;
  byte v;

  if (addr < 0 || addr >= maxDevices)
    return;
  if (digit < 0 || digit > 7 || value > 15)
    return;
  offset = addr * 8;
  v = charTable[value];
  if (dp)
    v |= B10000000;
  status[offset + digit] = v;
  spiTransfer(addr, digit + 1, v);

}

void Number_Baseball_Robot::setChar(int addr, int digit, char value, boolean dp) {
  int offset;
  byte index, v;

  if (addr < 0 || addr >= maxDevices)
    return;
  if (digit < 0 || digit > 7)
    return;
  offset = addr * 8;
  index = (byte)value;
  if (index > 127) {
    //nothing define we use the space char
    value = 32;
  }
  v = charTable[index];
  if (dp)
    v |= B10000000;
  status[offset + digit] = v;
  spiTransfer(addr, digit + 1, v);
}

void Number_Baseball_Robot::spiTransfer(int addr, volatile byte opcode, volatile byte data) {
  //Create an array with the data to shift out
  int offset = addr * 2;
  int maxbytes = maxDevices * 2;

  for (int i = 0; i < maxbytes; i++)
    spidata[i] = (byte)0;
  //put our device data into the array
  spidata[offset + 1] = opcode;
  spidata[offset] = data;
  //enable the line
  digitalWrite(LEDCONTROL_SPI_CS, LOW);
  //Now shift out the data
  for (int i = maxbytes; i > 0; i--)
    shiftOut(LEDCONTROL_SPI_MOSI, LEDCONTROL_SPI_CLK, MSBFIRST, spidata[i - 1]);
  //latch the data onto the display
  digitalWrite(LEDCONTROL_SPI_CS, HIGH);
}

int Number_Baseball_Robot::getCharArrayPosition(char input) {
  if ((input == ' ') || (input == '+')) return 10;
  if (input == ':') return 11;
  if (input == '-') return 12;
  if (input == '.') return 13;
  if ((input == '(')) return  14; //replace by '�'
  if ((input >= '0') && (input <= '9')) return (input - '0');
  if ((input >= 'A') && (input <= 'Z')) return (input - 'A' + 15);
  if ((input >= 'a') && (input <= 'z')) return (input - 'a' + 15);
  return 13;
}

void Number_Baseball_Robot::writeString(int mtx, char * displayString) {
  while ( displayString[0] != 0) {
    char c = displayString[0];
    int pos = getCharArrayPosition( c);
    displayChar(mtx, pos);
    delay(300);
    clearAll();
    displayString++;
  }
}


void Number_Baseball_Robot::displayChar(int matrix, int charIndex) {
  for (int i = 0; i < 6; i++) {
    setRow(matrix, i, alphabetBitmap[charIndex][i]);
  }
}